<?php

class Mlumen_Api_Descriptor {
	const MODEL = 'nn-ensemble-pl-lem';

	/**
	 * @var string
	 */
	private $api_url;

	/**
	 * @var Mlumen_Options
	 */
	private $api_options;

	/**
	 * @var array
	 */
	public $data;

	/**
	 * @var string
	 */
	public $error_message;


	public function __construct( string $api_url, Mlumen_Options $options ) {
		$this->api_url     = $api_url;
		$this->api_options = $options;
	}


	/**
	 * Suggest National Library of Poland Descriptors (NLPD) for a given text.
	 *
	 * @return bool
	 */
	public function get_suggest( string $text ) {
		$response = wp_remote_post( $this->api_url . '/projects/' . self::MODEL . '/suggest', array(
			'headers'     => [ 'Content-Type' => 'application/json' ],
			'body'        => json_encode( [ 'text' => $text, 'limit' => 100 ] ),
			'method'      => 'POST',
			'data_format' => 'body',
		) );

		if ( is_wp_error( $response ) ) {
			$this->error_message = __( 'Błąd połączenia', 'mlumen' ) . ' ' . $response->get_error_message();

			return false;
		}

		$body = json_decode( $response['body'] );
		$http_code = wp_remote_retrieve_response_code( $response );
		$http_message = wp_remote_retrieve_response_message( $response );
		if ( $http_code === 200 ) {

			if ( isset( $body->annif_result->results ) && is_array( $body->annif_result->results ) && count( $body->annif_result->results ) > 0 ) {
				$this->data = $this->prepare_response( $body->annif_result->results );

				return true;
			}

			$this->data = array();
			$this->error_message = __( 'Brak tagów', 'mlumen' );

		} elseif ( $http_code === 422 ) {
			$this->error_message = __( 'Błąd walidacji', 'mlumen' );

		} elseif ( $http_code === 500 ) {
			$this->error_message = __( 'Błąd wewnętrzny serwera', 'mlumen' );

		} elseif ( ! empty( $http_message ) ) {
			$this->error_message = $http_message . ' ( http code ' . $http_code . ' )';

		} else {
			$this->error_message = __( 'Błąd wewnętrzny', 'mlumen' ) . ' ( http code ' . $http_code . ' )';
		}

		return false;
	}

	/**
	 * Get descriptors, AJAX hook
	 */
	public function get_ajax_descriptors() {
		if ( isset( $_POST['text'] ) ) {
			$text = sanitize_text_field( $_POST['text'] );

			if ( $this->get_suggest( $text ) ) {
				wp_send_json( array(
					'tags'          => $this->data,
					'error_message' => $this->error_message
				) );
				wp_die();
			}

			wp_send_json( array(
				'tags'          => array(),
				'error_message' => $this->error_message
			) );
			wp_die();
		}
	}

	/**
	 * Find default ratio for tag size
	 *
	 * @return integer
	 */
	public function find_ratio( array $data ) {
		$fuse  = 10;
		$ratio = 0;

		while ( $fuse > 0 ) {

			foreach ( $data as $item ) {
				if ( ( 10 ** $ratio ) * $item > 1 ) {
					return $ratio;
				}
			}

			$ratio ++;
			$fuse --;
		}

		return $ratio;
	}

	/**
	 * Find minimum score
	 *
	 * @return float
	 */
	public function find_min_score( array $data ) {
		$min_score = reset( $data );

		foreach ( $data as $item ) {
			if ( $min_score > $item ) {
				$min_score = $item;
			}
		}

		return $min_score;
	}

	/**
	 * Prepare response
	 *
	 * @return array
	 */
	public function prepare_response( array $result ) {
		$font_ratio = $this->find_ratio( array_map( static function ( $item ) {
				return $item->score;
			}, $result )
		);

		$min_score = $this->find_min_score( array_map( static function ( $item ) use ( $font_ratio ) {
				return round( ( 10 ** $font_ratio ) * $item->score );
			}, $result )
		);

		$data = array_map( static function ( $item ) use ( $font_ratio, $min_score ) {
			return array(
				'name'     => esc_attr( str_replace( ',', '', $item->label ) ),
				'score'    => $item->score,
				'tag_size' => round( ( 10 ** $font_ratio ) * $item->score ) - $min_score
			);
		}, $result );

		// sort by score
		usort( $data, static function ( $a, $b ) {
			if ( $a['score'] === $b['score'] ) {
				return 0;
			}

			return ( $a['score'] < $b['score'] ) ? 1 : - 1;
		} );

		$minimal_tags_score = $this->api_options->options['minimal_tags_score'];
		$data = array_filter( $data, static function ( $item ) use ( $minimal_tags_score ) {
			return ( (float) $item['score'] >= (float) $minimal_tags_score );
		} );

		// set tags limit
		$data = array_slice( $data, 0, (int) $this->api_options->options['limit_tags'] );

		// sort alphabetically
		usort( $data, static function ( $a, $b ) {
			if ( $a['name'] === $b['name'] ) {
				return 0;
			}

			return ( $a['name'] < $b['name'] ) ? - 1 : 1;
		} );

		return $data;
	}

}